
library(plyr)
library(lattice)

rm(list=ls())
load("ResultTest.rpt_ALL")
load("ResultTrainCV.rpt_ALL")

#res = TestRes
res = TrainRes

colnames(res)[c(1, 6)] = c("comb", "p")
idx = sort(rep(c(1:432), 10))
res$idx = idx 

res.means = lapply(split(res, res$idx), function(x){
  colMeans(x, na.rm = TRUE)
})

res.sds = lapply(split(res, res$idx), function(x){
  apply(x[,8:12], 2, FUN = sd, na.rm = TRUE)
})

res.sds = data.frame(idx = 1:432, ldply(res.sds, .fun = rbind))[,-2]

res.means = ldply(res.means, .fun = rbind)
res.means = res.means[,-which(colnames(res.means) %in% c(".id", "Repeat"))]

resmeans_long <- reshape(res.means, timevar="Method",  v.names="Accuracy",
                          varying = list(c("PLDA","SVM","bagSVM","CART","RF")), times=c("PLDA","SVM","bagSVM","CART","RF"),
                          direction = "long")

ressds_long <- reshape(res.sds, timevar="Method",  v.names="Std.Dev.",
                     varying = list(c("PLDA","SVM","bagSVM","CART","RF")), times=c("PLDA","SVM","bagSVM","CART","RF"),
                     direction = "long")

plotData = data.frame(idx = resmeans_long$idx, resmeans_long[,-which(colnames(resmeans_long) %in% c("id", "idx"))], 
                      Std.Dev. = ressds_long$Std.Dev.)

lower = plotData$Accuracy - 1.96*plotData$Std.Dev./sqrt(plotData$n)
upper = plotData$Accuracy + 1.96*plotData$Std.Dev./sqrt(plotData$n)

upper[upper >= 1] = 1
lower[lower <= 0] = 0

plotData$Lower = lower
plotData$Upper = upper
rownames(plotData) = NULL

for (i in c(3:6,8)) plotData[,i] = factor(plotData[,i])
#plotData$Method = factor(plotData$Method)

##### GRAPHS

my.panel2 <- function(x, y, subscripts, col, pch, group.number, ...) {
  low95 <- my.subset$Lower[subscripts]
  up95 <- my.subset$Upper[subscripts]
  myjitter <- c(-4, -2, 0, 2, 4)
  panel.xyplot(x + myjitter[group.number], y, col=col, pch=pch, ...)
  panel.arrows(x+myjitter[group.number], low95, x+myjitter[group.number], up95, angle=90, code=3, 
               length=0.025, col=col, lwd=2)
}

my.panel <- function(x, y, subscripts, col, pch, group.number, ...) {
  low95 <- my.subset$Lower[subscripts]
  up95 <- my.subset$Upper[subscripts]
  myjitter <- c(-4, -2, 0, 2, 4)
  panel.xyplot(x + myjitter[group.number], y, col=col, pch=pch, ...)
  #panel.arrows(x+myjitter[group.number], low95, x+myjitter[group.number], up95, angle=90, code=3, length=0.05, col=col)
}

# n <- c(20,40,60,80)
# p <- c(25,50,75,99)
# kg <- c(2,3,4)
# param <- c(100,10,1)  ## nbinom() fonksiyonunda 1/param olarak dikkate al??n??yor. dolay??s??yla, param (1,10,100)
# sdsignal <- 3
# DE = c(0.01, 0.05, 0.10)
# P.all = 10000 ## Total number of features.

DE = 0.1
k = 4
met = "VST"

my.subset = plotData[(plotData[,"kg"] == k & plotData[,"DERate"] == DE),]
#my.subset = plotData[(plotData[,"kg"]==2 & plotData[,"param"]==100 & plotData[,"DERate"]==0.01),]
#my.subset = EXP.MEANS_long[(EXP.MEANS_long[,"kg"]==4 & EXP.MEANS_long[,"param"]==0.1),]
#apply(my.subset[,3:8], 2, function(x)table(x))

my.subset = my.subset[order(my.subset[,"Method"]),]
panel.labels = paste("n = ", c(20,40,60,80), sep="")[rep(c(1:4),3)]

xyplot(Accuracy~p|factor(param:n), data=my.subset, strip = strip.custom(factor.levels = panel.labels), scales = list(cex = 2),
       groups=Method, type="b",lty=1:5, col=c(1,3,2,4,6), layout=c(4,3), pch=10:14, panel.groups="my.panel2", 
       panel = "panel.superpose", ylim=c(0,1.05), ylab = list(cex = 2), xlab = list(cex=2, label = "Number of genes"),
       lwd=4, xlim = c(10,110), cex=1.7, main=paste("DERate = ", DE, "   nClass = ", k,  "   Method = ", met, sep=""))











xyplot(Accuracy~p|factor(n, labels=c("n = 20","n = 40","n = 60", "n = 80")), data=my.subset, groups=Method, type="b",
       lty=1:5, col=c(1,3,2,4,6), layout=c(4,1), pch=10:14, panel.groups="my.panel", panel = "panel.superpose", ylim=c(0,1.05), 
       lwd=1.5, xlim = c(10,110), cex=0.6, main="deneme",
       key=list(x=.02, y=0.95, corner=c(0,1), text=list(levels(my.subset[,"Method"]), cex=.8), points=list(pch=c(10:14), col=c(1,3,2,4,6), cex=.9), 
                title=expression(bold("Method")), cex.title=.8))




### ANIL'dan alınan kodlar..

library(foreign)
require(lattice) 
library(lattice)

sps <- trellis.par.get("superpose.symbol")
sps$pch <- c(1,2,22,8)
sps$cex<-c(1.2,1.2,1.2,1.2)
sps$col <- c("black","black","black","black")
sps$fill<-c("white","white","white","white")
trellis.par.set("superpose.symbol", sps)
bck<-trellis.par.get("strip.background")
bck$col <- c("#D9D7D7","#A6A4A4","black","black")
trellis.par.set("strip.background", bck)

xyplot(FT1~TEST|Variance*Tau, data=x, groups=Depen, relation="sliced",
       scales=list(x=list(rot=90, cex=0.8)), ylab="Familywise Type I Error Rate", xlab="Nonparametric Multiple Comparison Tests", main="LS3E",
       auto.key = list(space = "right", reverse.rows= FALSE, title=expression(bold("Dependency")), cex.title=0.8, points = TRUE),
       panel = function(...){
         panel.abline(h=0.05, lty = 3, col = "black", lwd=2)
         panel.xyplot(...)
       })