
baggedSVM <- function(tr, cl, ts = NULL, cl.ts = NULL, nFolds = 5, repeats = 10, B = 10, 
                      m, combs.sub, foldIdx = NULL, seed = NULL, ...){
    
    if(!is.null(seed)){set.seed(seed)}
    cl = as.factor(cl); 
    if(!is.null(cl.ts)){cl.ts = factor(cl.ts, levels = levels(cl))}
    
    nn = length(cl)
    foldAccuracy = NULL
    tsAccuracy = NULL

    for (rr in 1:repeats){
        ## indices of the samples in the folds.
        
        if (is.null(foldIdx)){
            yfoldsInOK = FALSE
            while (!yfoldsInOK){
                foldsIn = createFolds(1:nn, k = nFolds, list = TRUE, returnTrain = TRUE)
                foldsOut = lapply(foldsIn, function(x)c(1:nn)[-x])
                yfoldsIn = lapply(foldsIn, FUN = function(x)cl[x])
                yfoldsOut = lapply(foldsIn, FUN = function(x)cl[-x])
                
                yfoldsInOK = all(ldply(lapply(yfoldsIn, FUN = function(x)length(unique(x)) == combs.sub[m,"kg"]))[,2])
            }
        }
        
        if (!is.null(foldIdx)){
            tmpp = c(1, seq(nFolds, nFolds*repeats, by = nFolds) + 1)
            foldsIn = foldIdx$indexIn[tmpp[rr]:(tmpp[rr+1] - 1)]
            foldsOut = foldIdx$indexOut[tmpp[rr]:(tmpp[rr+1] - 1)]
            yfoldsIn = lapply(foldsIn, FUN = function(x)cl[x])
            yfoldsOut = lapply(foldsIn, FUN = function(x)cl[-x])
        }
        
        preds = factor(rep(NA, nn), levels = levels(cl))
        tsAccuracy.fold = NULL
        for (i in 1:length(foldsIn)){
            x = tr[foldsIn[[i]],]
            y = cl[foldsIn[[i]]]
            xts = tr[foldsOut[[i]],]
            yts = cl[foldsOut[[i]]]
           
            baggPred_Matrix = foreach(j = 1:B, .multicombine = FALSE, .inorder = TRUE, .combine = "cbind", 
                                      .export = c("x", "y", "xts", "yts", "combs.sub", "m", "fitModel", "predictModel"),
                                      .packages = c("caret")) %dopar% {
                                          
                                          ## Her bir Ã§ekirdeÄŸe gÃ¶nderilen iÅŸlemlerin caret tarafÄ±ndan tekrar paralel olarak kullanÄ±lmaya zorlanmasÄ± nedeniyle RAM kapasitesinde gereksiz
                                          ## kullanÄ±m ortaya Ã§Ä±kÄ±yor. Foreach dÃ¶ngÃ¼sÃ¼ iÃ§inde caret tarafÄ±ndan tekrar paralel iÅŸlem yapÄ±lmasÄ±nÄ± engellemek gerekiyor. allowParallel = FALSE
                                          ## olarak ayarlanmalÄ±.
                                          ctrl2 <- trainControl(method = "repeatedcv", number = 2, repeats = 2, allowParallel = FALSE)  ## Paralel kodlama foreach aÅŸamasÄ±nda veriliyor.
                                          #ctrl2 <- trainControl(allowParallel = FALSE) 
                                          
                                          n2 = nrow(x)
                                          predictedSVM.B_ts = NULL
                                          BOOT.OK = FALSE
                                          while (!BOOT.OK){
                                              idx = sort(sample(n2, n2, TRUE))
                                              x.B = x[idx,]; y.B = y[idx]
                                              BOOT.OK <- (length(unique(y.B)) == combs.sub[m,"kg"])
                                          }
                                          
                                          modelSVM.B = fitModel(x = x.B, y = as.factor(y.B), trControl = ctrl2, 
                                                              tuneLength = 5, method = "svmRadial")

                                          if (class(modelSVM.B) == "train"){
                                              predictedSVM.B = predict.train(modelSVM.B, newdata = xts)   ## prediction for fold-out samples
                                          }
                                          if (class(modelSVM.B) != "train"){
                                              predictedSVM.B = rep(NA, nrow(xts))   ## prediction for fold-out samples
                                          }
                                          
                                          if (!is.null(ts)){
                                              if (class(modelSVM.B) == "train"){
                                                predictedSVM.B_ts = predict.train(modelSVM.B, newdata = ts)  ## prediction for independent test set
                                              }
                                              if (class(modelSVM.B) != "train"){
                                                  predictedSVM.B_ts = rep(NA, nrow(ts))  ## prediction for independent test set
                                              }
                                          }
                                          c(predictedSVM.B, predictedSVM.B_ts)
                                      }
            
            AggrBaggPred = factor(baggAgregate(baggPred_Matrix, levels = NULL), levels = levels(cl))  ## aggregated results for fold-out and independent test samples.
            predsFold.k = factor(AggrBaggPred[1:length(yts)], levels = levels(cl))    ## aggregated results for fold-out samples.    
            if (!is.null(ts)){
                predTest.k = factor(AggrBaggPred[(length(yts) + 1):length(AggrBaggPred)], levels = levels(cl)) ## aggregated results for independent test samples.
                tsACC.k = sum(cl.ts == predTest.k) / length(predTest.k)
                tsAccuracy.fold = c(tsAccuracy.fold, tsACC.k )
            }
            preds[foldsOut[[i]]] = predsFold.k
        }
        
        foldAccuracy.r = sum(preds == cl) / length(cl)
        foldAccuracy = c(foldAccuracy, foldAccuracy.r)
        if (!is.null(ts)) tsAccuracy = c(tsAccuracy, mean(tsAccuracy.fold))
    }
    
    return(list(Fold.Accuracy.Train = foldAccuracy, OveralAccuracy.Train = mean(foldAccuracy), 
                Fold.Accuracy.Test = tsAccuracy, OveralAccuracy.Test = mean(tsAccuracy)))
}
