

library(PoiClaClu)
source("baggAggregate.R")

B = 101
ctrl <- trainControl(method = "repeatedcv", number=5, repeats = 10)
p = c(25,50,75)
data(cervical)
transform = "vst"
rpt = 1

data = t(cervical)
cond = factor(rep(c(1,2), c(29,29)))
data = cbind(data, cond)

train.ind = sample(1:nrow(data), ceiling(0.70*nrow(data)), FALSE)
test.ind = c(1:nrow(data))[-train.ind]

train.raw = data[train.ind, ]
test.raw = data[-train.ind, ]

nZ = nearZeroVar(train.raw[, -dim(train.raw)[2]])

if(length(nZ) != 0){
  train.raw = train.raw[,-nZ]
  test.raw = test.raw[,-nZ]      
}

train.cond = train.raw[,dim(train.raw)[2]]
test.cond = test.raw[,dim(train.raw)[2]]
classes = factor(c(1,2))

#############  DESeq2
train.rawS4 = DESeqDataSetFromMatrix(countData = as.matrix(t(train.raw[, -dim(train.raw)[2]] + 1)), 
                                     colData = data.frame(factor(train.cond)), formula(~ 1))
design(train.rawS4) = ~ 1
dds = DESeq(train.rawS4)
res = results(dds)
#resOrdered <- res[order(res$padj),]
resOrdered <- res[order(res$stat, decreasing = TRUE),]

p.ind = 1
feat.Selected = rownames(as.data.frame(resOrdered)[1:p[p.ind],])

trPLDA <- tr <- train.raw[, feat.Selected]   ## PLDA'da doğrudan bu matrix kullanılacak.
tsPLDA <- ts <- test.raw[, feat.Selected]
#trExp = dataexpTrain[, feat.Selected]
#tsExp = 

#################################

tr = t(tr + 1)
ts = t(ts + 1)

#Calculation of test set size factors using geometric means from training data
geomts = ts / exp(rowMeans(log(tr)))
sizeF.ts = apply(geomts, 2, function(x) median(x))

#deseq normalization and vst transformation to training data
trS4 = DESeqDataSetFromMatrix(countData = as.matrix(tr), colData = data.frame(condition=factor(train.cond)), formula(~ 1))
design(trS4) = ~ 1
trS4 = estimateSizeFactors(trS4)
trS4 = estimateDispersions(trS4, fitType=c("local"))
trS4expVST = DESeq2:::varianceStabilizingTransformation(trS4, blind = FALSE)  ## DESeq2 ye göre güncellenecek (yukarıdakiler)
trS4expRLOG = rlog(trS4, blind = F)
dataexpTrainVST = t(assay(trS4expVST))
dataexpTrainRLOG = t(assay(trS4expRLOG))
betaPriorVar = attr(trS4expRLOG,"betaPriorVar")
intercept = mcols(trS4expRLOG)$rlogIntercept

#deseq normalization and vst transformation to test data
tsS4 = DESeqDataSetFromMatrix(countData = as.matrix(ts), colData = data.frame(condition=factor(test.cond)), formula(~ 1))
tsS4 = estimateSizeFactors(tsS4)
sizeFactors(tsS4) = sizeF.ts #Test set size factors are calculated above
dispersionFunction(tsS4) = dispersionFunction(trS4)
mcols(tsS4)$dispFit <- mcols(trS4)$dispFit
tsS4expVST = DESeq2:::varianceStabilizingTransformation(tsS4, blind = F)
tsS4expRLOG = rlog(tsS4, blind = F, intercept=intercept, betaPriorVar=betaPriorVar)
tsVST <- dataexpTestVST <- t(assay(tsS4expVST))
tsRLOG <- dataexpTestRLOG <- t(assay(tsS4expRLOG))

trVST = dataexpTrainVST
trRLOG = dataexpTrainRLOG
#################################

# tr.Param değerleri expression verileri için uygulanacak.
######tr.ParamVST = list(Mean = colMeans(trVST), Std.Dev. = apply(trVST,2,sd))
######tr.ParamRLOG = list(Mean = colMeans(trRLOG), Std.Dev. = apply(trRLOG,2,sd))
#trExp.Param = list(Mean = colMeans(trExp), Std.Dev. = apply(trExp,2,sd))

## Test set tr.Param değerlerine göre 
######tsVST = scale(tsVST, center = tr.ParamVST$Mean, scale = tr.ParamVST$Std.Dev.)
######tsRLOG = scale(tsRLOG, center = tr.ParamRLOG$Mean, scale = tr.ParamRLOG$Std.Dev.)

if (transform == "vst") tr = trVST else tr = trRLOG
if (transform == "vst") ts = tsVST else ts = tsRLOG

modelRF = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                    tuneLength = 10, method = "rf"))
predictedRF = predict.train(modelRF, newdata = ts)    
ts.accRF = sum(test.cond == predictedRF) / length(test.cond)

## CART
modelCART = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                      tuneLength = 10, method = "rpart"))
predictedCART = predict.train(modelCART, newdata = ts)    
ts.accCART = sum(test.cond == predictedCART) / length(test.cond)

## SVM
modelSVM = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                     tuneLength = 10, method = "svmRadial"))
predictedSVM = predict.train(modelSVM, newdata = ts)    
ts.accSVM = sum(test.cond == predictedSVM) / length(test.cond)

## bagSVM
#baggPred_Matrix = data.frame(matrix(NA, nrow = nrow(ts), ncol = B))
print("Bagging SVM is in progress...")
baggPred_Matrix = foreach(i = 1:B, .multicombine = TRUE, .inorder = FALSE, .combine = "cbind", 
                          .export = c("tr", "ts", "train.cond", "test.cond", "ctrl"),
                          .packages = c("caret")) %dopar% {
                            n = nrow(tr)
                            idx = sample(n, n, TRUE)
                            tr.B = tr[idx,]; train.cond.B = train.cond[idx]
                            
                            modelSVM.B = try(train(tr.B, as.factor(train.cond.B), trControl=ctrl, 
                                                   tuneLength = 10, method = "svmRadial"))
                            predictedSVM.B = predict.train(modelSVM.B, newdata = ts)    
                            predictedSVM.B        
                          }

predictedbaggSVM = baggAgregate(baggPred_Matrix, classes)
ts.accbaggSVM = sum(test.cond == predictedbaggSVM) / length(test.cond)

#######################
ts.accPDA <-ts.accPDA.mean <- NULL
#  Poisson Discriminant without transformation.
for (r in 1:rpt){
  cv.outRAW <- Classify.cv(trPLDA, train.cond, transform=FALSE, type = "deseq")
  outRAW <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW$bestrho, type = "deseq")
  tblRAW <- table(outRAW$ytehat,test.cond)
  ts.accPDA.r <- sum(diag(tblRAW))/sum(tblRAW)
  ts.accPDA = c(ts.accPDA, ts.accPDA.r)
}
ts.accPDA.mean = mean(ts.accPDA)

ts.accCART; ts.accRF
ts.accSVM; ts.accbaggSVM
ts.accPDA.mean
