library(MLSeq)
library(DESeq2)

#Creation of example training and test sets
data(cervical)
tr = cervical[1:15, c(1:5, 30:34)]
ts = cervical[1:15, c(6:7, 35:36)]

tr = tr + 1
ts = ts + 1

train.cond = rep(1:2, c(5,5))
test.cond = rep(1:2, c(2,2))

#Calculation of test set size factors using geometric means from training data
geomts = ts / exp(rowMeans(log(tr)))
sizeF.ts = apply(geomts, 2, function(x) median(x))

#deseq normalization and vst transformation to training data
trS4 = DESeqDataSetFromMatrix(countData = as.matrix(tr), colData = data.frame(condition=train.cond), formula(~ 1))
design(trS4) = ~ 1
trS4 = estimateSizeFactors(trS4)
trS4 = estimateDispersions(trS4, fitType=c("local"))
trS4expVST = varianceStabilizingTransformation(trS4, blind = F)
trS4expRLOG = rlog(trS4, blind = F)
dataexpTrainVST = t(assay(trS4expVST))
dataexpTrainRLOG = t(assay(trS4expRLOG))
betaPriorVar = attr(trS4expRLOG,"betaPriorVar")
intercept = mcols(trS4expRLOG)$rlogIntercept

#deseq normalization and vst transformation to test data
tsS4 = DESeqDataSetFromMatrix(countData = as.matrix(ts), colData = data.frame(condition=test.cond), formula(~ 1))
tsS4 = estimateSizeFactors(tsS4)
sizeFactors(tsS4) = sizeF.ts #Test set size factors are calculated above
dispersionFunction(tsS4) = dispersionFunction(trS4)
mcols(tsS4)$dispFit <- mcols(trS4)$dispFit
tsS4expVST = varianceStabilizingTransformation(tsS4, blind = F)
tsS4expRLOG = rlog(tsS4, blind = F, intercept=intercept, betaPriorVar=betaPriorVar)
dataexpTestVST = t(assay(tsS4expVST))
dataexpTestRLOG = t(assay(tsS4expRLOG))

#plotPCA(trS4expVST)
#plotPCA(trS4expRLOG)
#plotPCA(tsS4expVST)
#plotPCA(tsS4expRLOG)

