library(caret)
library(e1071)

B = 6

idx = sort(sample(150, 150*.7, FALSE))
tr = iris[idx, -5]
ts = iris[-idx, -5]
cl = iris[idx, 5]
cl.ts = iris[-idx, 5]

#tr = iris[c(51:65, 101:115), -5]
#ts = iris[c(66:75, 116:122), -5]
#cl = as.factor(as.character(iris[c(51:65, 101:115), 5]))
#cl.ts = as.factor(as.character(iris[c(66:75, 116:122), 5]))

ctrl = trainControl(method = "repeatedcv", number = 2, repeats = 1, verboseIter = TRUE)

svm = train(tr, cl, method = "svmRadial", trControl = ctrl, tuneLength = 10)
svm = train(tr, cl, method = "svmRadial", tuneLength = 10, prob.model = TRUE,
            trControl = trainControl(method = "boot", number = 25))

bagSVM = train(tr, cl, method = "bag", trControl = ctrl, tuneLength = 10, B = B,
               bagControl = bagControl(fit = svmBag$fit,
                                       predict = svmBag$pred,
                                       aggregate = svmBag$aggregate))

model = ksvm(as.matrix(tr), cl, kernel = "rbfdot", C = as.numeric(svm$bestTune[2]), prob.model = TRUE,
              kpar = list(sigma = as.numeric(svm$bestTune[1])))


pred = predict(model, tr)
pred2 = predict.train(svm, tr)
  
table(pred, pred2)


############
source("baggAggregate.R")

data = iris
#data = gokmencount(n = 20, p = 25, K = 4, param = 1, sdsignal = 3, DE = 0.1)

n = nrow(data)
nFolds = 5; rep = 2
B = 6

tr = data[, -dim(data)[2]]
cl = as.factor(data[, dim(data)[2]])

#cl = data[, 5]
#tr = data[, -5]

ctrl = trainControl("repeatedcv", 5, 2)

#start = Sys.time()
bagSVM = train(tr, cl, method = "bag", trControl = ctrl, tuneLength = 10, B = B,
               bagControl = bagControl(fit = svmBag$fit,
                                       predict = svmBag$pred,
                                       aggregate = svmBag$aggregate))
#end = Sys.time()
#end - start

start = Sys.time()

foldAccuracy = NULL
for (r in 1:rep){
  ## indices of the samples in the folds.
  foldsIn = createFolds(1:n, k = nFolds, list = TRUE, returnTrain = TRUE)
  foldsOut = lapply(foldsIn, function(x)c(1:n)[-x])
  
  yfoldsIn = lapply(foldsIn, FUN = function(x)cl[x])
  yfoldsOut = lapply(foldsIn, FUN = function(x)cl[-x])
  
  yfoldsInOK = all(ldply(lapply(yfoldsIn, FUN = function(x)length(unique(x)) == 3))[,2])
  
  preds = NULL  
  for (i in 1:length(foldsIn)){
    x = tr[foldsIn[[i]],]
    y = cl[foldsIn[[i]]]
    xts = tr[-foldsIn[[i]],]
    yts = cl[-foldsIn[[i]]]
    
    baggPred_Matrix = foreach(j = 1:B, .multicombine = TRUE, .inorder = FALSE, .combine = "cbind", 
                              .export = c("x", "y", "xts", "yts"),
                              .packages = c("caret")) %dopar% {
                                
                                ## Her bir çekirdeğe gönderilen işlemlerin caret tarafından tekrar paralel olarak kullanılmaya zorlanması nedeniyle RAM kapasitesinde gereksiz
                                ## kullanım ortaya çıkıyor. Foreach döngüsü içinde caret tarafından tekrar paralel işlem yapılmasını engellemek gerekiyor. allowParallel = FALSE
                                ## olarak ayarlanmalı.
                                ctrl2 <- trainControl(method = "repeatedcv", number = 2, repeats = 2, allowParallel = FALSE)  ## Paralel kodlama foreach aşamasında veriliyor.
                                n = nrow(x)
                                
                                BOOT.OK = FALSE
                                while (!BOOT.OK){
                                  idx = sort(sample(n, n, TRUE))
                                  x.B = x[idx,]; y.B = y[idx]
                                  BOOT.OK <- (length(unique(y.B)) == 3)
                                }
                                
                                modelSVM.B = try(train(x.B, as.factor(y.B), trControl=ctrl2, 
                                                       tuneLength = 5, method = "svmRadial"))
                                
                                predictedSVM.B = predict.train(modelSVM.B, newdata = xts)
                                #predTrain.B = predict.train(modelSVM.B, x)
                                #accTrain = sum() / length(y)
                                #c(predictedSVM.B, max(modelSVM.B$results[3]))
                                predictedSVM.B
                      }
    
    foldBaggPred = baggAgregate(baggPred_Matrix, levels = NULL)
    preds[foldsOut[[i]]] = foldBaggPred
  }
  
  preds = as.factor(levels(cl)[preds])
  foldAccuracy.r = sum(preds == cl) / length(cl)
  foldAccuracy = c(foldAccuracy, foldAccuracy.r)
}

end = Sys.time()
end - start
