rm(list=ls())

#library(PoiClaClu)
library(DESeq2)
library(caret)
library(kernlab)
library(randomForest)

## Parallel Programming Background

library(doSNOW)   ## W??NDOWS i??in
library(doParallel)
library(doMC)   ## MAC ve LINUX i??in
library(foreach)
library(digest)
source("baggAggregate.R")

## Source codes of PLDA (Witten et. al.)
source("Classify.cv.R")
source("Classify.R")
source("CountDataSet.R")
source("FindBestTransform.R")
source("Functions.R")
source("NullModel.R")
source("NullModelTest.R")
source("PoissonDistance.R")


#multicore:::detectCores()  ## LINUX ve MAC i??in kullan??labilecek maksimum ??ekirdek say??s??n?? belirler.

#cl = makeCluster(8, type="SOCK")
#registerDoSNOW(cl)

#registerDoMC(4)  ## Kullan??lacak ??ekirdek say??s??n??n belirlenmesi. (MAC ve LINUX i??in

#registerDoParallel(6)

continue = FALSE  ## elektrik kesintisi durum
m.last = 1  ## Bilgisayar??n kapanmas?? durumunda en son hangi noktada kald??????n?? g??steren indis.

B = 101
rpt = 10  ## number of repeat.
ctrl <- trainControl(method = "repeatedcv", number=5, repeats = 10)
transform = "vst"   # vst veya rlog 

n <- c(20,40,60,80)
p <- c(25,50,75,99)
kg <- c(2,3,4)
param <- c(100,10,1)  ## nbinom() fonksiyonunda 1/param olarak dikkate al??n??yor. dolay??s??yla, param (1,10,100)
sdsignal <- 3
DE = c(0.01, 0.05, 0.10)
P.all = 10000 ## Total number of features.

combs = expand.grid(n,kg,param,DE)  ## Olas?? kombinasyonlar?? tutan matris. Her bir kolonda ilgili vekt??r??n elemanlar??n?? sakl??yor. (data frame)
n.comb = dim(combs)[1]  ## number of possible combinations
id = 1:n.comb

combs = cbind(id,combs)
colnames(combs) = c("id","n","kg","param","DERate")

#combs.sub = combs[c(70,37),]  ## Analiz edilecek kombinasyonlar belirleniyor.
n.comb.sub = dim(combs.sub)[1]  ## number of possible combinations of subset of combs


gokmencount<-function (n, p, K, param, sdsignal, DE = 0.3)
{
  if (n < 4 * K) 
    stop("We require n to be at least 4*K.")
  q0 <- rexp(p, rate = 1/25)
  isDE <- runif(p) < DE  ## %the probability of a gene to be differentially expressed.  DE = c(2, 10, 30)
  classk <- matrix(NA, nrow = K, ncol = p)
  for (k in 1:K) {
    lfc <- rnorm(p, sd = sdsignal)
    classk[k, ] <- ifelse(isDE, q0 * exp(lfc), q0)
  }
  truesf <- runif(n) * 2 + 0.2
  truesfte <- runif(n) * 2 + 0.2
  conds <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
  condste <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
  x <- xte <- matrix(NA, nrow = n, ncol = p)
  for (i in 1:n) {
    for (k in 1:K) {
      if (conds[i] == k) 
        x[i, ] <- rnbinom(p, mu = truesf[i] * classk[k,], size = param)
      if (condste[i] == k)
        xte[i, ] <- rnbinom(p, mu = truesfte[i] * classk[k,], size = param)
    }
  }
  rm <- apply(x, 2, sum) == 0
  y <- conds
  gok <- as.data.frame(cbind(x[, !rm],y))
  return(gok)
}


data <- NULL
dataexpTrain <- NULL

PLDAresultTest.rpt <- list()
ResultTest.rpt <- list()
ResultTrainCV.rpt <- list()

TuneParameters.rpt = list()
RAWdataSets.rpt = list()


if (continue) {
  load("TestResults.rda")
  load("CVResults.rda")
  load("TuneParameters.rda")
}

if (!continue) {m.last = 1}
#set.seed(280985)

## m indisi 432 olas?? kombinasyon i??in kullan??lan bir indis. Hangi parametreler ile veri uretilip simulasyon yap??ld??????n?? kontrol ediyor.

start.time = Sys.time()

for (m in m.last:n.comb.sub){
  
  classes = as.factor(c(1:combs.sub[m,"kg"]))
  classes = classes[1]
  ## Her bir olas?? kombinasyon i??in rpt kadar tekrar yap??lacak.
  testACC_m = list() 
  trainACC_CV_m = list()
  tunes_m = list()
  for (p.ind in 1:length(p)){
    
    # CART, RF, SVM, bagSVM, PLDA
    testACC = matrix(NA, nrow=rpt, ncol=6)
    trainACC_CV = matrix(NA, nrow=rpt, ncol=6)
    colnames(testACC) = colnames(trainACC_CV) = c("Repeat", "PLDA", "SVM", "bagSVM", "CART", "RF")
    tunes = NULL
    
    testACC[,1] = trainACC_CV[,1] = 1:rpt
    for (R in 1:rpt){
      
      progress = paste("Comb.: ", m, "/", n.comb.sub, "    p: ", p[p.ind], "    Repeat: ", R, "/", rpt, sep="")
      print(progress)
      
      ## RAW data ??retilir.
      ## Üretilen RAW data'da sınıf değişkeni istenilen kadar sınıfa sahip olmalı.
      ## kg = 4 iken, n = 20 olan veri seti 1,2, ve 4 numaralı sınıfları içeriyor fakat 3'ü içermiyor ise
      ## bu veri setinin yenidnen üretilmesi gerekiyor. Üretilen veri tam olarak kg adet sınıfı içermelidir.
      nCl.OK = FALSE
      
      while(!nCl.OK){
        data <- gokmencount(n=combs.sub[m,"n"], p=P.all, K=combs.sub[m,"kg"], 
                            param=combs.sub[m,"param"], sdsignal=sdsignal, DE = combs.sub[m,"DERate"])
        nCl.OK <- length(unique(data$y)) == combs.sub[m,"kg"]
      }
      
      trts.OK = FALSE
      while (!trts.OK){
        train.ind = sample(1:nrow(data), ceiling(0.70*nrow(data)), FALSE)
        test.ind = c(1:nrow(data))[-train.ind]
        train.raw = data[train.ind, ]
        test.raw = data[-train.ind, ]
        
        # train setindeki sınıf sayısının ilgili kombinasyondaki sınıf sayısı kadar olması gerekiyor. 4 sınıflı bir
        # kombinasyonda üretilen veride yalnızca 1,2 ve 4 sınıfları var ise verinin yeniden test / train olarak ayrılması gerekiyor.
        ## Önemli olan train kümseinde istenilen sayıda sınıfın yer alması. Test setinde sınıflardan bir tanesi yer almasa dahi
        # classifier'lar 4 sınıfa göre eğitildiği için sınıflama performansları bu durumdan olumsuz etkilenmeyecektir.
        trts.OK <- (length(unique(train.raw$y)) == combs.sub[m,"kg"])
      }
        
      nZ = nearZeroVar(train.raw[, -dim(train.raw)[2]])
  
      if(length(nZ) != 0){
        train.raw = train.raw[,-nZ]
        test.raw = test.raw[,-nZ]      
      }
      
      train.cond = train.raw[,dim(train.raw)[2]]
      test.cond = test.raw[,dim(train.raw)[2]]
      
      #############  DESeq2
      train.rawS4 = DESeqDataSetFromMatrix(countData = as.matrix(t(train.raw[, -dim(train.raw)[2]] + 1)), 
                                           colData = data.frame(factor(train.cond)), formula(~ 1))
      design(train.rawS4) = ~ 1
      dds = DESeq(train.rawS4, fitType=c("local"))
      res = results(dds)
      #resOrdered <- res[order(res$padj),]
      resOrdered <- res[order(res$stat, decreasing = TRUE),]
      feat.Selected = rownames(as.data.frame(resOrdered)[1:p[p.ind],])

      trPLDA <- tr <- train.raw[, feat.Selected]   ## PLDA'da do??rudan bu matrix kullan??lacak.
      tsPLDA <- ts <- test.raw[, feat.Selected]
      #trExp = dataexpTrain[, feat.Selected]
      #tsExp = 
      
      #################################
            
      tr = t(tr + 1)
      ts = t(ts + 1)
      
      #Calculation of test set size factors using geometric means from training data
      geomts = ts / exp(rowMeans(log(tr)))
      sizeF.ts = apply(geomts, 2, function(x) median(x))
      
      #deseq normalization and vst transformation to training data
      trS4 = DESeqDataSetFromMatrix(countData = as.matrix(tr), colData = data.frame(condition=factor(train.cond)), formula(~ 1))
      design(trS4) = ~ 1
      trS4 = estimateSizeFactors(trS4)
      trS4 = estimateDispersions(trS4, fitType=c("local"))
      trS4expVST = DESeq2:::varianceStabilizingTransformation(trS4, blind = FALSE)  ## DESeq2 ye g??re g??ncellenecek (yukar??dakiler)
      trS4expRLOG = rlog(trS4, blind = F)
      dataexpTrainVST = t(assay(trS4expVST))
      dataexpTrainRLOG = t(assay(trS4expRLOG))
      betaPriorVar = attr(trS4expRLOG,"betaPriorVar")
      intercept = mcols(trS4expRLOG)$rlogIntercept
      
      #deseq normalization and vst transformation to test data
      tsS4 = DESeqDataSetFromMatrix(countData = as.matrix(ts), colData = data.frame(condition=factor(test.cond)), formula(~ 1))
      tsS4 = estimateSizeFactors(tsS4)
      sizeFactors(tsS4) = sizeF.ts    #Test set size factors are calculated above
      dispersionFunction(tsS4) = dispersionFunction(trS4)  ## Use dispersion parameters from training set
      mcols(tsS4)$dispFit <- mcols(trS4)$dispFit
      tsS4expVST = DESeq2:::varianceStabilizingTransformation(tsS4, blind = F)
      tsS4expRLOG = rlog(tsS4, blind = F, intercept=intercept, betaPriorVar=betaPriorVar)
      tsVST <- dataexpTestVST <- t(assay(tsS4expVST))
      tsRLOG <- dataexpTestRLOG <- t(assay(tsS4expRLOG))
      
      trVST = dataexpTrainVST
      trRLOG = dataexpTrainRLOG
      #################################
      
      # tr.Param de??erleri expression verileri i??in uygulanacak.
      ######tr.ParamVST = list(Mean = colMeans(trVST), Std.Dev. = apply(trVST,2,sd))
      ######tr.ParamRLOG = list(Mean = colMeans(trRLOG), Std.Dev. = apply(trRLOG,2,sd))
      #trExp.Param = list(Mean = colMeans(trExp), Std.Dev. = apply(trExp,2,sd))
      
      ## Test set tr.Param de??erlerine g??re 
      ######tsVST = scale(tsVST, center = tr.ParamVST$Mean, scale = tr.ParamVST$Std.Dev.)
      ######tsRLOG = scale(tsRLOG, center = tr.ParamRLOG$Mean, scale = tr.ParamRLOG$Std.Dev.)
      
      if (transform == "vst") tr = trVST else tr = trRLOG
      if (transform == "vst") ts = tsVST else ts = tsRLOG
      
      ## SVM
      modelSVM = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                       tuneLength = 10, method = "svmRadial"))
      predictedSVM = predict.train(modelSVM, newdata = ts)    
      ts.accSVM = sum(test.cond == as.numeric(predictedSVM)) / length(test.cond)
      
      ## bagSVM
      #baggPred_Matrix = data.frame(matrix(NA, nrow = nrow(ts), ncol = B))
      print("Bagging SVM is in progress...")
      baggPred_Matrix = foreach(i = 1:B, .multicombine = TRUE, .inorder = FALSE, .combine = "cbind", 
                       .export = c("tr", "ts", "train.cond", "test.cond","combs.sub"),
                       .packages = c("caret")) %dopar% {
        
        ## Her bir çekirdeğe gönderilen işlemlerin caret tarafından tekrar paralel olarak kullanılmaya zorlanması nedeniyle RAM kapasitesinde gereksiz
        ## kullanım ortaya çıkıyor. Foreach döngüsü içinde caret tarafından tekrar paralel işlem yapılmasını engellemek gerekiyor. allowParallel = FALSE
        ## olarak ayarlanmalı.
        ctrl2 <- trainControl(method = "repeatedcv", number=5, repeats = 10, allowParallel = FALSE)  ## Paralel kodlama foreach aşamasında veriliyor.
        n = nrow(tr)
        
        BOOT.OK = FALSE
        while (!BOOT.OK){
          idx = sample(n, n, TRUE)
          tr.B = tr[idx,]; train.cond.B = train.cond[idx]
          BOOT.OK <- (length(unique(train.cond.B)) == combs.sub[m,"kg"])
        }
        
        modelSVM.B = try(train(tr.B, as.factor(train.cond.B), trControl=ctrl2, 
                               tuneLength = 10, method = "svmRadial"))
        predictedSVM.B = predict.train(modelSVM.B, newdata = ts)    
        c(predictedSVM.B, max(modelSVM.B$results[3]))
      }
      
      predictedbaggSVM = baggAgregate(baggPred_Matrix[-nrow(baggPred_Matrix),], classes)
      ts.accbaggSVM = sum(test.cond == as.numeric(predictedbaggSVM)) / length(test.cond)
      
      ## RF
      modelRF = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                       tuneLength = 10, method = "rf"))
      predictedRF = predict.train(modelRF, newdata = ts)    
      ts.accRF = sum(test.cond == as.numeric(predictedRF)) / length(test.cond)
      
      ## CART
      modelCART = try(train(tr, as.factor(train.cond), trControl=ctrl, 
                      tuneLength = 10, method = "rpart"))
      predictedCART = predict.train(modelCART, newdata = ts)    
      ts.accCART = sum(test.cond == as.numeric(predictedCART)) / length(test.cond)
      
      #######################
      ts.accPDA <-ts.accPDA.mean <- trACC.cv <- NULL
      #  Poisson Discriminant without transformation.
      try({
        for (r in 1:10){
          cv.outRAW <- Classify.cv(trPLDA, train.cond, transform=FALSE, type = "deseq")
          outRAW <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW$bestrho, type = "deseq")
          tblRAW <- table(outRAW$ytehat,test.cond)
          ts.accPDA.r <- sum(diag(tblRAW))/sum(tblRAW)
          ts.accPDA = c(ts.accPDA, ts.accPDA.r)
          
          ind.opt = which(cv.outRAW$rhos == cv.outRAW$bestrho)
          trErr.cv = cv.outRAW$errs[,ind.opt]
          accCV = 1 - (sum(trErr.cv)/length(train.cond))
          trACC.cv = c(trACC.cv, accCV)
        }
      })
      
      ts.accPDA.mean = mean(ts.accPDA)
      
      ######################
      
      testACC[R, "SVM"] = ts.accSVM
      testACC[R, "RF"] = ts.accRF
      testACC[R, "CART"] = ts.accCART
      testACC[R, "PLDA"] = ts.accPDA.mean
      testACC[R, "bagSVM"]= ts.accbaggSVM
      
      trainACC_CV[R, "SVM"] = max(modelSVM$results$Accuracy)
      trainACC_CV[R, "PLDA"] = mean(trACC.cv)
      trainACC_CV[R, "RF"] = max(modelRF$results$Accuracy)
      trainACC_CV[R, "CART"] = max(modelCART$results$Accuracy)
      trainACC_CV[R, "bagSVM"] = mean(baggPred_Matrix[nrow(baggPred_Matrix),])
      #paste(c(names(combs.sub)[-1],"p"), c(combs.sub[1,-1],p[p.ind]), sep="", collapse = "")
      
      tunes.rpt = cbind(R, modelSVM$bestTune, modelCART$bestTune, modelRF$bestTune)
      tunes = rbind(tunes, tunes.rpt)
      tunes.rpt = NULL
    }

    trainACC_CV = try({cbind(combs.sub[m,], p[p.ind], trainACC_CV)}, silent = TRUE)
    testACC = try({cbind(combs.sub[m,], p[p.ind], testACC)}, silent = TRUE)
    tunes = try({cbind(combs.sub[m,], p[p.ind], tunes)}, silent = TRUE)
    
    tunes_m[[p.ind]] = tunes
    testACC_m[[p.ind]] = testACC  
    trainACC_CV_m[[p.ind]] = trainACC_CV 
    names(testACC_m)[p.ind] = names(trainACC_CV_m)[p.ind] = names(tunes_m)[p.ind] = paste("p", p[p.ind], sep="")  
  }
  
  ResultTest.rpt[[m]] = testACC_m
  ResultTrainCV.rpt[[m]] = trainACC_CV_m
  TuneParameters.rpt[[m]] = tunes_m
  names(TuneParameters.rpt)[m] = names(ResultTest.rpt)[m] = names(ResultTrainCV.rpt)[m] = names(TuneParameters.rpt)[m] = paste(c(names(combs.sub)[-1]), c(combs.sub[m,-1]), sep="", collapse = "")

  save(ResultTest.rpt, file = "TestResults.rda")
  save(ResultTrainCV.rpt, file = "CVResults.rda")
  save(TuneParameters.rpt, file = "TuneParameters.rda")
  
  halt = paste("Algorithm stopped at: ", m, sep = "")
  write.table(halt, file = "stopped.txt", quote = FALSE, row.names = FALSE, col.names = FALSE)
}

end.time = Sys.time()
elapsed =  end.time - start.time
elapsed   ## elapsed time for simulation.