rm(list=ls())

#library(PoiClaClu)
library(DESeq2)
library(caret)
library(kernlab)
library(randomForest)
library(sSeq)
library(plyr)

## Parallel Programming Background

#library(doSNOW)   ## W??NDOWS i??in
#library(doParallel)
library(doMC)   ## MAC ve LINUX i??in
library(foreach)
library(digest)
#library(doRNG)  ## paralel kodlama seed özelliğini aktif etmek için

source("allFunctions.R")

## Source codes of PLDA (Witten et. al.)
source("Classify.cv.R")
source("Classify.R")
source("CountDataSet.R")
source("FindBestTransform.R")
source("Functions.R")
source("NullModel.R")
source("NullModelTest.R")
source("PoissonDistance.R")

## Source codes of NBLDA
source("classnb.R")
source("CountDataSet1.R")
source("GetDnb.R")


#multicore:::detectCores()  ## LINUX ve MAC i??in kullan??labilecek maksimum ??ekirdek say??s??n?? belirler.

#cl = makeCluster(8, type="SOCK")
#registerDoSNOW(cl)

registerDoMC(6)  ## Kullan??lacak ??ekirdek say??s??n??n belirlenmesi. (MAC ve LINUX i??in

#registerDoParallel(6)

wd = getwd()  ## working directory.

save = TRUE  ## save results.
continue = FALSE  ## elektrik kesintisi durum
m.last = 1  ## Bilgisayar??n kapanmas?? durumunda en son hangi noktada kald??????n?? g??steren indis.

B = 101
rpt = 10  ## number of repeat. (whole scenario)
repeats = 10; nFolds = 5

#ctrl <- trainControl(method = "repeatedcv", number=5, repeats = 10)
transform = "vst"   # vst veya rlog 

#n=c(80); p=50; kg=c(2); param=100; sdsignal=3; DE=.1; P.all=10000

n <- c(20,40,60,80)
p <- c(25,50,75,99)
kg <- c(2,3,4)
param <- c(100,10,1)  ## nbinom() fonksiyonunda 1/param olarak dikkate al??n??yor. dolay??s??yla, param (1,10,100)
sdsignal <- 3
DE = c(0.01, 0.05, 0.10)
P.all = 10000 ## Total number of features.

combs = expand.grid(n,kg,param,DE)  ## Olas?? kombinasyonlar?? tutan matris. Her bir kolonda ilgili vekt??r??n elemanlar??n?? sakl??yor. (data frame)
n.comb = dim(combs)[1]  ## number of possible combinations
id = 1:n.comb

combs = cbind(id,combs)
colnames(combs) = c("id","n","kg","param","DERate")

combs.sub = combs[100:108,]  ## Analiz edilecek kombinasyonlar belirleniyor.
n.comb.sub = dim(combs.sub)[1]  ## number of possible combinations of subset of combs


gokmencount<-function (n, p, K, param, sdsignal, DE = 0.3)
{
  if (n < 4 * K) 
    stop("We require n to be at least 4*K.")
  q0 <- rexp(p, rate = 1/25)
  isDE <- runif(p) < DE  ## %the probability of a gene to be differentially expressed.  DE = c(2, 10, 30)
  classk <- matrix(NA, nrow = K, ncol = p)
  for (k in 1:K) {
    lfc <- rnorm(p, sd = sdsignal)
    classk[k, ] <- ifelse(isDE, q0 * exp(lfc), q0)
  }
  truesf <- runif(n) * 2 + 0.2
  truesfte <- runif(n) * 2 + 0.2
  conds <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
  condste <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
  x <- xte <- matrix(NA, nrow = n, ncol = p)
  for (i in 1:n) {
    for (k in 1:K) {
      if (conds[i] == k) 
        x[i, ] <- rnbinom(p, mu = truesf[i] * classk[k,], size = param)
      if (condste[i] == k)
        xte[i, ] <- rnbinom(p, mu = truesfte[i] * classk[k,], size = param)
    }
  }
  rm <- apply(x, 2, sum) == 0
  y <- conds
  gok <- as.data.frame(cbind(x[, !rm],y))
  return(gok)
}


data <- NULL
dataexpTrain <- NULL

PLDAresultTest.rpt <- list()
ResultTest.rpt <- list()
ResultTrainCV.rpt <- list()

TuneParameters.rpt = list()
RAWdataSets.rpt = list()


if (continue) {
  load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "TestResults.rda", sep = ""))
  load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "CVResults.rda", sep = ""))
  load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "TuneParameters.rda", sep = ""))
}

if (!continue) {m.last = 1}
set.seed(10072013)
#registerDoRNG(seed = 10072013)
#m <- p.ind <- R <- 1
## m indisi 432 olas?? kombinasyon i??in kullan??lan bir indis. Hangi parametreler ile veri uretilip simulasyon yap??ld??????n?? kontrol ediyor.

start.time = Sys.time()

for (m in m.last:n.comb.sub){
  
  classes = as.factor(c(1:combs.sub[m,"kg"]))
  classes = classes[1]
  ## Her bir olas?? kombinasyon i??in rpt kadar tekrar yap??lacak.
  testACC_m = list() 
  trainACC_CV_m = list()
  tunes_m = list()
  
  for (p.ind in 1:length(p)){
    
    # CART, RF, SVM, bagSVM, PLDA
    testACC = matrix(NA, nrow=rpt, ncol=8)
    trainACC_CV = matrix(NA, nrow=rpt, ncol=8)
    colnames(testACC) = colnames(trainACC_CV) = c("Repeat", "PLDA", "PLDA2", "NBLDA", "SVM", "bagSVM", "CART", "RF")
    tunes = NULL
    
    testACC[,1] = trainACC_CV[,1] = 1:rpt
    
    for (R in 1:rpt){
      progress = paste("Comb.: ", m, "/", n.comb.sub, "    p: ", p[p.ind], "    Repeat: ", R, "/", rpt, sep="")
      print(progress)
      
      ## RAW data ??retilir.
      ## Üretilen RAW data'da sınıf değişkeni istenilen kadar sınıfa sahip olmalı.
      ## kg = 4 iken, n = 20 olan veri seti 1,2, ve 4 numaralı sınıfları içeriyor fakat 3'ü içermiyor ise
      ## bu veri setinin yenidnen üretilmesi gerekiyor. Üretilen veri tam olarak kg adet sınıfı içermelidir.
      nCl.OK = FALSE
      
      while(!nCl.OK){
        data <- gokmencount(n=combs.sub[m,"n"], p=P.all, K=combs.sub[m,"kg"], 
                            param=combs.sub[m,"param"], sdsignal=sdsignal, DE = combs.sub[m,"DERate"])
        nCl.OK <- length(unique(data$y)) == combs.sub[m,"kg"]
      }
      
      trts.OK = FALSE
      while (!trts.OK){
        train.ind = sample(1:nrow(data), ceiling(0.70*nrow(data)), FALSE)
        test.ind = c(1:nrow(data))[-train.ind]
        train.raw = data[train.ind, ]
        test.raw = data[-train.ind, ]
        
        # train setindeki sınıf sayısının ilgili kombinasyondaki sınıf sayısı kadar olması gerekiyor. 4 sınıflı bir
        # kombinasyonda üretilen veride yalnızca 1,2 ve 4 sınıfları var ise verinin yeniden test / train olarak ayrılması gerekiyor.
        ## Önemli olan train kümseinde istenilen sayıda sınıfın yer alması. Test setinde sınıflardan bir tanesi yer almasa dahi
        # classifier'lar 4 sınıfa göre eğitildiği için sınıflama performansları bu durumdan olumsuz etkilenmeyecektir.
        trts.OK <- (length(unique(train.raw$y)) == combs.sub[m,"kg"])
      }
        
      nZ = nearZeroVar(train.raw[, -dim(train.raw)[2]])
  
      if(length(nZ) != 0){
        train.raw = train.raw[,-nZ]
        test.raw = test.raw[,-nZ]      
      }
      
      train.cond = train.raw[,dim(train.raw)[2]]
      test.cond = test.raw[,dim(train.raw)[2]]
      
      #############  DESeq2
      train.rawS4 = DESeqDataSetFromMatrix(countData = as.matrix(t(train.raw[, -dim(train.raw)[2]] + 1)), 
                                           colData = data.frame(factor(train.cond)), formula(~ 1))
      design(train.rawS4) = ~ 1
      dds = DESeq(train.rawS4, fitType=c("local"))
      res = results(dds)
      #resOrdered <- res[order(res$padj),]
      resOrdered <- res[order(res$stat, decreasing = TRUE),]
      feat.Selected = rownames(as.data.frame(resOrdered)[1:p[p.ind],])

      trPLDA <- tr <- train.raw[, feat.Selected]   ## PLDA'da do??rudan bu matrix kullan??lacak.
      tsPLDA <- ts <- test.raw[, feat.Selected]
      #trExp = dataexpTrain[, feat.Selected]
      #tsExp = 
      
      #################################
            
      tr = t(tr + 1)
      ts = t(ts + 1)
      
      #Calculation of test set size factors using geometric means from training data
      geomts = ts / exp(rowMeans(log(tr)))
      sizeF.ts = apply(geomts, 2, function(x) median(x))
      
      #deseq normalization and vst transformation to training data
      trS4 = DESeqDataSetFromMatrix(countData = as.matrix(tr), colData = data.frame(condition=factor(train.cond)), formula(~ 1))
      design(trS4) = ~ 1
      trS4 = estimateSizeFactors(trS4)
      trS4 = estimateDispersions(trS4, fitType=c("local"))
      
      if (transform == "vst"){
          trS4expVST = DESeq2:::varianceStabilizingTransformation(trS4, blind = FALSE)  ## DESeq2 ye g??re g??ncellenecek (yukar??dakiler)
          dataexpTrainVST = t(assay(trS4expVST))
      }
      
      if (transform == "rlog"){
          trS4expRLOG = rlog(trS4, blind = F)
          dataexpTrainRLOG = t(assay(trS4expRLOG))
          betaPriorVar = attr(trS4expRLOG,"betaPriorVar")
          intercept = mcols(trS4expRLOG)$rlogIntercept
      }
      
      #deseq normalization and vst transformation to test data
      tsS4 = DESeqDataSetFromMatrix(countData = as.matrix(ts), colData = data.frame(condition=factor(test.cond)), formula(~ 1))
      tsS4 = estimateSizeFactors(tsS4)
      sizeFactors(tsS4) = sizeF.ts    #Test set size factors are calculated above
      dispersionFunction(tsS4) = dispersionFunction(trS4)  ## Use dispersion parameters from training set
      mcols(tsS4)$dispFit <- mcols(trS4)$dispFit
      
      if (transform == "vst"){
          tsS4expVST = DESeq2:::varianceStabilizingTransformation(tsS4, blind = F)
          tsVST <- dataexpTestVST <- t(assay(tsS4expVST))
          trVST = dataexpTrainVST
      }
      
      if (transform == "rlog"){
          tsS4expRLOG = rlog(tsS4, blind = F, intercept=intercept, betaPriorVar=betaPriorVar)
          tsRLOG <- dataexpTestRLOG <- t(assay(tsS4expRLOG))
          trRLOG = dataexpTrainRLOG
      }

      #################################
      
      # tr.Param de??erleri expression verileri i??in uygulanacak.
      ######tr.ParamVST = list(Mean = colMeans(trVST), Std.Dev. = apply(trVST,2,sd))
      ######tr.ParamRLOG = list(Mean = colMeans(trRLOG), Std.Dev. = apply(trRLOG,2,sd))
      #trExp.Param = list(Mean = colMeans(trExp), Std.Dev. = apply(trExp,2,sd))
      
      ## Test set tr.Param de??erlerine g??re 
      ######tsVST = scale(tsVST, center = tr.ParamVST$Mean, scale = tr.ParamVST$Std.Dev.)
      ######tsRLOG = scale(tsRLOG, center = tr.ParamRLOG$Mean, scale = tr.ParamRLOG$Std.Dev.)
      
      if (transform == "vst") tr = trVST else tr = trRLOG
      if (transform == "vst") ts = tsVST else ts = tsRLOG
      
      fold.IdxOK = FALSE
      while (!fold.IdxOK){
          fold.Idx = foldIndex(tr, nFolds = nFolds, repeats = repeats)
          fold.Idx_In = fold.Idx$indexIn
          yfold.Idx_In = lapply(fold.Idx_In, FUN = function(x)as.factor(train.cond)[x])
          fold.IdxOK = all(ldply(lapply(yfold.Idx_In, FUN = function(x)length(unique(x)) == combs.sub[m,"kg"]))[,2])   
      }
      
      ctrl <- trainControl(method = "repeatedcv", number = nFolds, repeats = repeats,
                           index = fold.Idx$indexIn, indexOut = fold.Idx$indexOut)
      ## SVM
      modelSVM = fitModel(x = tr, y = as.factor(train.cond), trControl = ctrl, 
                          tuneLength = 10, method = "svmRadial")
      
      ts.accSVM = predictModel(modelSVM, ts)
      
      ## bagSVM
      #baggPred_Matrix = data.frame(matrix(NA, nrow = nrow(ts), ncol = B))
      #print("Bagging SVM is in progress...")
      #baggPred_Matrix = foreach(i = 1:B, .multicombine = TRUE, .inorder = FALSE, .combine = "cbind", 
      #                 .export = c("tr", "ts", "train.cond", "test.cond","combs.sub"),
      #                 .packages = c("caret")) %dopar% {
        
        ## Her bir çekirdeğe gönderilen işlemlerin caret tarafından tekrar paralel olarak kullanılmaya zorlanması nedeniyle RAM kapasitesinde gereksiz
        ## kullanım ortaya çıkıyor. Foreach döngüsü içinde caret tarafından tekrar paralel işlem yapılmasını engellemek gerekiyor. allowParallel = FALSE
        ## olarak ayarlanmalı.
      #  ctrl2 <- trainControl(method = "repeatedcv", number=5, repeats = 10, allowParallel = FALSE)  ## Paralel kodlama foreach aşamasında veriliyor.
      #  n = nrow(tr)
        
      #  BOOT.OK = FALSE
      #  while (!BOOT.OK){
      #    idx = sample(n, n, TRUE)
      #    tr.B = tr[idx,]; train.cond.B = train.cond[idx]
      #    BOOT.OK <- (length(unique(train.cond.B)) == combs.sub[m,"kg"])
      #  }
        
      #  modelSVM.B = try(train(tr.B, as.factor(train.cond.B), trControl=ctrl2, 
      #                         tuneLength = 10, method = "svmRadial"))
      #  predictedSVM.B = predict.train(modelSVM.B, newdata = ts)    
      #  c(predictedSVM.B, max(modelSVM.B$results[3]))
      #}
      
      #predictedbaggSVM = baggAgregate(baggPred_Matrix[-nrow(baggPred_Matrix),], classes)
      #ts.accbaggSVM = sum(test.cond == as.numeric(predictedbaggSVM)) / length(test.cond)
      
      print("Bagging SVM is in progress...")
      svm.B = try({baggedSVM(tr, as.factor(train.cond), ts, as.factor(test.cond), foldIdx = fold.Idx,
                        nFolds = nFolds, repeats = repeats, B = B, 
                        m = m, combs.sub = combs.sub)})
      
      #svm.B <- try(train(tr, as.factor(train.cond), "bag", B = B, 
      #                bagControl = bagControl(fit = svmBag$fit,
      #                                        predict = svmBag$pred,
      #                                        aggregate = svmBag$aggregate), 
      #                trControl = ctrl, tuneLength = 10))
      
      #predictedSVM.B = predict.train(svm.B, newdata = ts)
      #ts.accbaggSVM = sum(test.cond == as.numeric(predictedSVM.B)) / length(test.cond)
      
      ## RF
      modelRF = fitModel(x = tr, y = as.factor(train.cond), trControl=ctrl, 
                       tuneLength = 10, method = "rf")
      
      ts.accRF = predictModel(modelRF, newdata = ts)
      
      ## CART
      modelCART = fitModel(x = tr, y = as.factor(train.cond), trControl=ctrl, 
                      tuneLength = 10, method = "rpart")
      ts.accCART = predictModel(modelCART, ts)
      
      #######################
      ts.accPDA <- ts.accPDA.mean <- trACC.cv <- accCV <- accCV2 <- NULL
      ts.accPDA2 <-ts.accPDA2.mean <- trACC2.cv <- NULL
      trACCcv_NBLDA.repeats = NULL
      #  PLDA and NBLDA.
      ###  NBLDA ve PLDA foldları diğer yöntemlerde kullanılan foldların 1. repeat için bulunanları olacak şekilde alınacak.
      foldNB_PLDA = fold.Idx$indexOut[1:nFolds]
      try({
        #for (r in 1:10){
          cv.outRAW <- Classify.cv(trPLDA, train.cond, transform = FALSE, type = "deseq", folds = foldNB_PLDA)
          outRAW <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW$bestrho, type = "deseq")
          #tblRAW <- table(outRAW$ytehat,test.cond)
          ts.accPDA <- sum(outRAW$ytehat == test.cond) / length(test.cond)
          #ts.accPDA = c(ts.accPDA, ts.accPDA.r)
          
          ind.opt = which(cv.outRAW$rhos == cv.outRAW$bestrho)
          trErr.cv = cv.outRAW$errs[,ind.opt]
          accCV = 1 - sum(trErr.cv) / length(train.cond)
          #trACC.cv = c(trACC.cv, accCV)
          
          cv.outRAW2 <- Classify.cv(trPLDA, train.cond, transform = TRUE, type = "deseq", folds = cv.outRAW$folds)
          outRAW2 <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW2$bestrho, type = "deseq")
          #tblRAW2 <- table(outRAW2$ytehat, test.cond)
          ts.accPDA2 <- sum(outRAW2$ytehat == test.cond) / length(test.cond)
          #ts.accPDA2 = c(ts.accPDA2, ts.accPDA2.r)
          
          ind.opt2 = which(cv.outRAW2$rhos == cv.outRAW2$bestrho)
          trErr2.cv = cv.outRAW2$errs[,ind.opt2]
          accCV2 = 1 - sum(trErr2.cv) / length(train.cond)
          #trACC2.cv = c(trACC2.cv, accCV2)
          
          ##### NBLDA
          # Creating folds (5 folds)
          # NBLDA için 5 fold yapmayı gerektirecek bir durum yok.
          # Optimize edilmesi gereken bir parametre olmadığı için tek aşamada modelin eğitilip 
          # train ve test setlerinin tahmin edilmesi gerekiyor..
          
          #nn = nrow(trPLDA)
          #yfoldsInOK = FALSE
          
         # while (!yfoldsInOK){
          #  foldsIn = createFolds(1:nn, k = 5, list = TRUE, returnTrain = TRUE)
         #   foldsOut = lapply(foldsIn, function(x)c(1:nn)[-x])
         #   yfoldsIn = lapply(foldsIn, FUN = function(x)train.cond[x])
         #   yfoldsOut = lapply(foldsIn, FUN = function(x)train.cond[-x])
         #   yfoldsInOK = all(ldply(lapply(yfoldsIn, FUN = function(x)length(unique(x)) == combs.sub[m, "kg"]))[,2])  ### combs.sub'dan gelecek olan kg ile kontrol edilecek.        
         # }
          
          #trACCcv_NBLDA.folds = NULL
          #tsACCcv_NBLDA.folds = NULL
          #predsNB.tr = rep(NA, length(train.cond))
         
          #foldsIn = fold.Idx$indexIn[1:nFolds]
          #foldsOut = fold.Idx$indexOut[1:nFolds]
         
          #for (ii in 1:length(foldsIn)){
            trNB = trPLDA
            y.trNB = train.cond
            #trNB = trPLDA[foldsIn[[ii]],]
            #y.trNB = train.cond[foldsIn[[ii]]]
            #tsNB = trPLDA[foldsOut[[ii]],]
            #y.tsNB = train.cond[foldsOut[[ii]]]
            
            tt = getT(t(trNB), sizeFactors = rep(1, nrow(trNB)))$target
            rM = rowMeans(t(trNB))
            rV = rowVars(t(trNB))
            
            disp = (rV - rM)/rM^2
            disp0 = numeric()
            
            for(i in 1:length(disp)){
              a1 = 0; a2 = disp[i]
              disp0[i] = max(a1,a2)
            }
            
            disperhat = getAdjustDisp(disp0, shrinkTarget = tt)$adj
            out = classnb(trNB, y.trNB, trNB, type = "deseq")
            out.ts = classnb(trNB, y.trNB, tsPLDA, type = "deseq")
            #predsNB.tr[foldsOut[[ii]]] = out$ytehat
            #trACCcv_NBLDA.ii = sum(out$ytehat == y.tsNB) / length(y.tsNB)
            #trACCcv_NBLDA.folds = c(trACCcv_NBLDA.folds, trACCcv_NBLDA.ii)
            #tsACCcv_NBLDA.ii = sum(out.ts$ytehat == test.cond) / length(test.cond)
            #tsACCcv_NBLDA.folds = c(tsACCcv_NBLDA.folds, tsACCcv_NBLDA.ii)
          #}
          #trAccNB.train = sum(predsNB.tr == train.cond) / length(train.cond)
          trAccNB.train = sum(out$ytehat == train.cond) / length(train.cond)
          #trACCcv_NBLDA.repeats = c(trACCcv_NBLDA.repeats, mean(trACCcv_NBLDA.folds))
          #tsAccNB.test = mean(tsACCcv_NBLDA.folds, na.rm = TRUE)
          tsAccNB.test = sum(out.ts$ytehat == test.cond) / length(test.cond)
        #} ## end for r
      })
      
      #ts.accPDA.mean = mean(ts.accPDA)
      #ts.accPDA2.mean = mean(ts.accPDA2)
      #tr.accNBLDA.mean = mean(trAccNB.train)  ## 5 fold NBLDA cv accuracy
      
      
      ######################
      
      testACC[R, "SVM"] = ts.accSVM
      testACC[R, "RF"] = ts.accRF
      testACC[R, "CART"] = ts.accCART
      testACC[R, "PLDA"] = ts.accPDA
      testACC[R, "PLDA2"] = ts.accPDA2
      #testACC[R, "bagSVM"] = ts.accbaggSVM
      testACC[R, "bagSVM"] = svm.B$OveralAccuracy.Test
      testACC[R, "NBLDA"] = tsAccNB.test
      
      if (class(modelSVM) == "train") trainACC_CV[R, "SVM"] = max(modelSVM$results$Accuracy) else trainACC_CV[R, "SVM"] = NA
      if (class(modelRF) == "train") trainACC_CV[R, "RF"] = max(modelRF$results$Accuracy) else trainACC_CV[R, "RF"] = NA
      if (class(modelCART) == "train") trainACC_CV[R, "CART"] = max(modelCART$results$Accuracy) else trainACC_CV[R, "CART"] = NA
      trainACC_CV[R, "PLDA"] = accCV
      trainACC_CV[R, "PLDA2"] = accCV2
      trainACC_CV[R, "NBLDA"] = trAccNB.train
      trainACC_CV[R, "bagSVM"] = svm.B$OveralAccuracy.Train
      
      #paste(c(names(combs.sub)[-1],"p"), c(combs.sub[1,-1],p[p.ind]), sep="", collapse = "")
      
      if (class(modelSVM) == "train") svmTune = modelSVM$bestTune else svmTune = NA
      if (class(modelCART) == "train") cartTune = modelCART$bestTune else cartTune = NA
      if (class(modelRF) == "train") rfTune = modelRF$bestTune else rfTune = NA
      
      tunes.rpt = cbind(R, svmTune, cartTune, rfTune)
      tunes = rbind(tunes, tunes.rpt)
      tunes.rpt = NULL
      
    }

    trainACC_CV = try({cbind(combs.sub[m,], p[p.ind], trainACC_CV)}, silent = TRUE)
    testACC = try({cbind(combs.sub[m,], p[p.ind], testACC)}, silent = TRUE)
    tunes = try({cbind(combs.sub[m,], p[p.ind], tunes)}, silent = TRUE)
    
    tunes_m[[p.ind]] = tunes
    testACC_m[[p.ind]] = testACC  
    trainACC_CV_m[[p.ind]] = trainACC_CV 
    names(testACC_m)[p.ind] = names(trainACC_CV_m)[p.ind] = names(tunes_m)[p.ind] = paste("p", p[p.ind], sep="")  
  }
  
  ResultTest.rpt[[m]] = testACC_m
  ResultTrainCV.rpt[[m]] = trainACC_CV_m
  TuneParameters.rpt[[m]] = tunes_m
  names(TuneParameters.rpt)[m] = names(ResultTest.rpt)[m] = names(ResultTrainCV.rpt)[m] = names(TuneParameters.rpt)[m] = paste(c(names(combs.sub)[-1]), c(combs.sub[m,-1]), sep="", collapse = "")

  if (save){
      if (transform == "rlog") {
          folders = dir()
          if (!any(folders == "RLOG")) dir.create("RLOG", recursive = TRUE)
          setwd("RLOG")
      }
      
      if (transform == "vst") {
          folders = dir()
          if (!any(folders == "VST")) dir.create("VST", recursive = TRUE)
          setwd("VST")
      }
      
      save(ResultTest.rpt, file = "TestResults.rda")
      save(ResultTrainCV.rpt, file = "CVResults.rda")
      save(TuneParameters.rpt, file = "TuneParameters.rda")
      
      halt = paste("Algorithm stopped at: ", m, sep = "")
      write.table(halt, file = "stopped.txt", quote = FALSE, row.names = FALSE, col.names = FALSE)
  }
  setwd(wd)
}

end.time = Sys.time()
elapsed =  end.time - start.time
elapsed   ## elapsed time for simulation.

