rm(list=ls())

###### Inputs  ################################################################################################

multComp = "doParallel"   ## Paralel proglamlama seçimi.  Windows için "doParallel" veya "doSNOW"
                          ## MAC ve Linux sistemler için "doMC" seçilmelidir.

nCores = 6                ## Paralel kullanılacak olan çekirdek sayısı.

selectedCombs = 1:2       ## Seçilen kombinasyonlar.

save = FALSE              ## Sonuçların kaydedilmesi için TRUE girilmeli. TRUE olarak bırakın.
continue = FALSE          ## Seçilen kombinasyonların tamamı bitmeden bilgisayar kapanmış ise, en son kalınan
                          ## noktadan devam edilecek ise TRUE girilmelidir.

m.last = 1                ## continue = TRUE ise en son kalınan kombinasyonun sıra numarası girilmelidir.

Seed = 10072013           ## Random Number Generator seed. 
B = 6
rpt = 2                   ## number of repeat. (whole scenario)
repeats = 2; nFolds = 2
#transform = c("vst", "rlog")         ## vst veya rlog 

n <- c(40,60,80,100)
p <- c(25,50,75,100)
kg <- c(2,3,4)
param <- c(100,10,1)      ## nbinom() fonksiyonunda 1/param olarak dikkate alınıyor. param: (1,10,100)
sdsignal <- 3
DE = c(0.01, 0.05, 0.10)
P.all = 10000             ## Total number of features.

###############################################################################################################

#library(PoiClaClu)
library(DESeq2)
library(caret)
library(kernlab)
library(randomForest)
library(sSeq)
library(plyr)

## Parallel Programming Background

if (multComp == "doSNOW") library(doSNOW)   ## W??NDOWS i??in
if (multComp == "doParallel") library(doParallel)
if (multComp == "doMC") library(doMC)   ## MAC ve LINUX i??in
library(foreach)
library(digest)
#library(doRNG)  ## paralel kodlama seed özelliğini aktif etmek için

source("allFunctions.R")

## Source codes of PLDA (Witten et. al.)
source("Classify.cv.R")
source("Classify.R")
source("CountDataSet.R")
source("FindBestTransform.R")
source("Functions.R")
source("NullModel.R")
source("NullModelTest.R")
source("PoissonDistance.R")

## Source codes of NBLDA
source("classnb.R")
source("CountDataSet1.R")
source("GetDnb.R")

#multicore:::detectCores()  ## LINUX ve MAC i??in kullan??labilecek maksimum ??ekirdek say??s??n?? belirler.

if (multComp == "doSNOW"){
    clusts = makeCluster(nCores, type="SOCK")
    registerDoSNOW(clusts)
}

if (multComp == "doMC") registerDoMC(nCores)
if (multComp == "doParallel") registerDoParallel(nCores)

wd = getwd()  ## working directory.
combs = expand.grid(n,kg,param,DE)  ## Olas?? kombinasyonlar?? tutan matris. Her bir kolonda ilgili vekt??r??n elemanlar??n?? sakl??yor. (data frame)
n.comb = dim(combs)[1]  ## number of possible combinations
id = 1:n.comb

combs = cbind(id,combs)
colnames(combs) = c("id","n","kg","param","DERate")

combs.sub = combs[selectedCombs,]  ## Analiz edilecek kombinasyonlar belirleniyor.
n.comb.sub = dim(combs.sub)[1]  ## number of possible combinations of subset of combs

gokmencount<-function (n, p, K, param, sdsignal, DE = 0.3)
{
    if (n < 4 * K) 
        stop("We require n to be at least 4*K.")
    q0 <- rexp(p, rate = 1/25)
    isDE <- runif(p) < DE  ## %the probability of a gene to be differentially expressed.  DE = c(2, 10, 30)
    classk <- matrix(NA, nrow = K, ncol = p)
    for (k in 1:K) {
        lfc <- rnorm(p, sd = sdsignal)
        classk[k, ] <- ifelse(isDE, q0 * exp(lfc), q0)
    }
    truesf <- runif(n) * 2 + 0.2
    truesfte <- runif(n) * 2 + 0.2
    conds <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
    condste <- sample(c(rep(1:K, 4), sample(1:K, n - 4 * K, replace = TRUE)))
    x <- xte <- matrix(NA, nrow = n, ncol = p)
    for (i in 1:n) {
        for (k in 1:K) {
            if (conds[i] == k) 
                x[i, ] <- rnbinom(p, mu = truesf[i] * classk[k,], size = param)
            if (condste[i] == k)
                xte[i, ] <- rnbinom(p, mu = truesfte[i] * classk[k,], size = param)
        }
    }
    rm <- apply(x, 2, sum) == 0
    y <- conds
    gok <- as.data.frame(cbind(x[, !rm],y))
    return(gok)
}


data <- NULL
dataexpTrain <- NULL

PLDAresultTest.rpt <- list()
ResultTest.rpt <- list()
ResultTrainCV.rpt <- list()

TuneParameters.rpt = list()
RAWdataSets.rpt = list()


if (continue) {
    #load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "TestResults.rda", sep = ""))
    #load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "CVResults.rda", sep = ""))
    #load(file = paste(ifelse(transform == "rlog", "RLOG//", "VST//"), "TuneParameters.rda", sep = ""))
    load(file = "VST_RLOG//TestResults.rda")
    load(file = "VST_RLOG//CVResults.rda")
    load(file = "VST_RLOG//TuneParameters.rda")
}

if (!continue) {m.last = 1}
set.seed(seed = Seed)
#registerDoRNG(seed = 10072013)
#m <- p.ind <- R <- 1
## m indisi 432 olas?? kombinasyon i??in kullan??lan bir indis. Hangi parametreler ile veri uretilip simulasyon yap??ld??????n?? kontrol ediyor.

start.time = Sys.time()

for (m in m.last:n.comb.sub){   ## Loop for each combination
    
    #classes = as.factor(c(1:combs.sub[m,"kg"]))
    classes = factor(1, levels = c(1:combs.sub[m,"kg"]))
    #classes = classes[1]
    ## Her bir olas?? kombinasyon i??in rpt kadar tekrar yap??lacak.
    testACC_m = list() 
    trainACC_CV_m = list()
    tunes_m = list()
    
    for (p.ind in 1:length(p)){  ## Loop for each number of features selected.
        
        # CART, RF, SVM, bagSVM, PLDA
        testACC = matrix(NA, nrow=2*rpt, ncol=9)
        trainACC_CV = matrix(NA, nrow=2*rpt, ncol=9)
        colnames(testACC) = colnames(trainACC_CV) = c("Transform" ,"Repeat", "PLDA", "PLDA2", "NBLDA", "SVM", "bagSVM", "CART", "RF")
        tunes = NULL
        
        testACC[,"Repeat"] = trainACC_CV[,"Repeat"] = rep(c(1:rpt), rep(2,rpt))
        testACC[,"Transform"] = trainACC_CV[,"Transform"] = rep(c(1,2), rpt)    ## transform --- 1: vst,  2: rlog
        
        for (R in 1:rpt){
            
                s4.OK <- FALSE
                s4.iter = 1
                while (!s4.OK && s4.iter < 5){
                    progress = paste("Comb.: ", m, "/", n.comb.sub, "    p: ", p[p.ind], "    Repeat: ", R, "/", rpt, sep="")
                    print(progress)
                    
                    ## RAW data ??retilir.
                    ## Üretilen RAW data'da sınıf değişkeni istenilen kadar sınıfa sahip olmalı.
                    ## kg = 4 iken, n = 20 olan veri seti 1,2, ve 4 numaralı sınıfları içeriyor fakat 3'ü içermiyor ise
                    ## bu veri setinin yenidnen üretilmesi gerekiyor. Üretilen veri tam olarak kg adet sınıfı içermelidir.
                    nCl.OK = FALSE
                    
                    while(!nCl.OK){
                        data <- gokmencount(n=combs.sub[m,"n"], p=P.all, K=combs.sub[m,"kg"], 
                                            param=combs.sub[m,"param"], sdsignal=sdsignal, DE = combs.sub[m,"DERate"])
                        nCl.OK <- length(unique(data$y)) == combs.sub[m,"kg"]
                    }
                    
                    trts.OK = FALSE
                    while (!trts.OK){
                        train.ind = sample(1:nrow(data), ceiling(0.70*nrow(data)), FALSE)
                        test.ind = c(1:nrow(data))[-train.ind]
                        train.raw = data[train.ind, ]
                        test.raw = data[-train.ind, ]
                        
                        # train setindeki sınıf sayısının ilgili kombinasyondaki sınıf sayısı kadar olması gerekiyor. 4 sınıflı bir
                        # kombinasyonda üretilen veride yalnızca 1,2 ve 4 sınıfları var ise verinin yeniden test / train olarak ayrılması gerekiyor.
                        ## Önemli olan train kümseinde istenilen sayıda sınıfın yer alması. Test setinde sınıflardan bir tanesi yer almasa dahi
                        # classifier'lar 4 sınıfa göre eğitildiği için sınıflama performansları bu durumdan olumsuz etkilenmeyecektir.
                        trts.OK <- (length(unique(train.raw$y)) == combs.sub[m,"kg"])
                    }
                    
                    nZ = nearZeroVar(train.raw[, -dim(train.raw)[2]])
                    
                    if(length(nZ) != 0){
                        train.raw = train.raw[,-nZ]
                        test.raw = test.raw[,-nZ]      
                    }
                    
                    train.cond = train.raw[,dim(train.raw)[2]]
                    test.cond = test.raw[,dim(train.raw)[2]]
                    
                    #############  DESeq2
                    train.rawS4 = try({DESeqDataSetFromMatrix(countData = as.matrix(t(train.raw[, -dim(train.raw)[2]] + 1)), 
                                                         colData = data.frame(factor(train.cond)), formula(~ 1))})
                    
                    if (class(train.rawS4) != "try-error") s4.OK <- TRUE
                    s4.iter = s4.iter + 1
                    if (s4.iter == 5) cat("Iteration does not converge. Stopped.")
                    if (!s4.OK) cat("Re-generating raw data since \"DESeqDataSetFromMatrix\" function generated errors.", sep = "")
                }
                
                design(train.rawS4) = ~ 1
                dds = DESeq(train.rawS4, fitType=c("local"))
                res = results(dds)
                #resOrdered <- res[order(res$padj),]
                resOrdered <- res[order(res$stat, decreasing = TRUE),]
                feat.Selected = rownames(as.data.frame(resOrdered)[1:p[p.ind],])
                
                trPLDA <- tr <- train.raw[, feat.Selected]   ## PLDA'da do??rudan bu matrix kullan??lacak.
                tsPLDA <- ts <- test.raw[, feat.Selected]
                #trExp = dataexpTrain[, feat.Selected]
                #tsExp = 
                
                #################################
                
                tr = t(tr + 1)
                ts = t(ts + 1)
                
                #Calculation of test set size factors using geometric means from training data
                geomts = ts / exp(rowMeans(log(tr)))
                sizeF.ts = apply(geomts, 2, function(x) median(x))
                
                #deseq normalization and vst transformation to training data
                trS4 = DESeqDataSetFromMatrix(countData = as.matrix(tr), colData = data.frame(condition=factor(train.cond)), formula(~ 1))
                design(trS4) = ~ 1
                trS4 = estimateSizeFactors(trS4)
                trS4 = estimateDispersions(trS4, fitType=c("local"))
                
                #deseq normalization and vst transformation to test data
                tsS4 = DESeqDataSetFromMatrix(countData = as.matrix(ts), colData = data.frame(condition=factor(test.cond)), formula(~ 1))
                tsS4 = estimateSizeFactors(tsS4)
                sizeFactors(tsS4) = sizeF.ts    #Test set size factors are calculated above
                dispersionFunction(tsS4) = dispersionFunction(trS4)  ## Use dispersion parameters from training set
                mcols(tsS4)$dispFit <- mcols(trS4)$dispFit
                
                fold.IdxOK = FALSE
                while (!fold.IdxOK){
                    fold.Idx = foldIndex(tr, nFolds = nFolds, repeats = repeats)
                    fold.Idx_In = fold.Idx$indexIn
                    yfold.Idx_In = lapply(fold.Idx_In, FUN = function(x)as.factor(train.cond)[x])
                    fold.IdxOK = all(ldply(lapply(yfold.Idx_In, FUN = function(x)length(unique(x)) == combs.sub[m,"kg"]))[,2])   
                }
                
                for (transform in c("vst", "rlog")){
					if (transform == "vst"){
						trS4expVST = DESeq2:::varianceStabilizingTransformation(trS4, blind = FALSE)  ## DESeq2 ye g??re g??ncellenecek (yukar??dakiler)
						dataexpTrainVST = t(assay(trS4expVST))
						tsS4expVST = DESeq2:::varianceStabilizingTransformation(tsS4, blind = F)
						tsVST <- dataexpTestVST <- t(assay(tsS4expVST))
						trVST = dataexpTrainVST
						tr <- trVST
						ts <- tsVST
						idxTransform <- 1
					} 
					
					if (transform == "rlog"){
						trS4expRLOG = rlog(trS4, blind = F)
						dataexpTrainRLOG = t(assay(trS4expRLOG))
						betaPriorVar = attr(trS4expRLOG,"betaPriorVar")
						intercept = mcols(trS4expRLOG)$rlogIntercept
						
						tsS4expRLOG = rlog(tsS4, blind = F, intercept=intercept, betaPriorVar=betaPriorVar)
						tsRLOG <- dataexpTestRLOG <- t(assay(tsS4expRLOG))
						trRLOG = dataexpTrainRLOG
						tr <- trRLOG
						ts <- tsRLOG
						idxTransform <- 2
					}
					
					#################################
					# tr.Param de??erleri expression verileri i??in uygulanacak.
					######tr.ParamVST = list(Mean = colMeans(trVST), Std.Dev. = apply(trVST,2,sd))
					######tr.ParamRLOG = list(Mean = colMeans(trRLOG), Std.Dev. = apply(trRLOG,2,sd))
					#trExp.Param = list(Mean = colMeans(trExp), Std.Dev. = apply(trExp,2,sd))
					
					## Test set tr.Param de??erlerine g??re 
					######tsVST = scale(tsVST, center = tr.ParamVST$Mean, scale = tr.ParamVST$Std.Dev.)
					######tsRLOG = scale(tsRLOG, center = tr.ParamRLOG$Mean, scale = tr.ParamRLOG$Std.Dev.)
					
					#if (transform == "vst") tr = trVST else tr = trRLOG
					#if (transform == "vst") ts = tsVST else ts = tsRLOG
										
					ctrl <- trainControl(method = "repeatedcv", number = nFolds, repeats = repeats,
										 index = fold.Idx$indexIn, indexOut = fold.Idx$indexOut)
					## SVM
					modelSVM = fitModel(x = tr, y = as.factor(train.cond), trControl = ctrl, 
										tuneLength = 10, method = "svmRadial")
					
					ts.accSVM = predictModel(modelSVM, ts)
					
					## bagSVM
					## Her bir çekirdeğe gönderilen işlemlerin caret tarafından tekrar paralel olarak kullanılmaya zorlanması nedeniyle RAM kapasitesinde gereksiz
					## kullanım ortaya çıkıyor. Foreach döngüsü içinde caret tarafından tekrar paralel işlem yapılmasını engellemek gerekiyor. allowParallel = FALSE
					## olarak ayarlanmalı.
					
					print("Bagging SVM is in progress...")
					svm.B = try({baggedSVM(tr, as.factor(train.cond), ts, as.factor(test.cond), foldIdx = fold.Idx,
										   nFolds = nFolds, repeats = repeats, B = B, 
										   m = m, combs.sub = combs.sub)})
		
					## RF
					modelRF = fitModel(x = tr, y = as.factor(train.cond), trControl=ctrl, 
									   tuneLength = 10, method = "rf")
					
					ts.accRF = predictModel(modelRF, newdata = ts)
					
					## CART
					modelCART = fitModel(x = tr, y = as.factor(train.cond), trControl=ctrl, 
										 tuneLength = 10, method = "rpart")
					ts.accCART = predictModel(modelCART, ts)
					
					#######################
					ts.accPDA <- ts.accPDA.mean <- trACC.cv <- accCV <- accCV2 <- NULL
					ts.accPDA2 <-ts.accPDA2.mean <- trACC2.cv <- NULL
					trACCcv_NBLDA.repeats = NULL
					#  PLDA and NBLDA.
					###  NBLDA ve PLDA foldları diğer yöntemlerde kullanılan foldların 1. repeat için bulunanları olacak şekilde alınacak.
					foldNB_PLDA = fold.Idx$indexOut[1:nFolds]
					try({
						cv.outRAW <- Classify.cv(trPLDA, train.cond, transform = FALSE, type = "deseq", folds = foldNB_PLDA)
						outRAW <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW$bestrho, type = "deseq")
						ts.accPDA <- sum(outRAW$ytehat == test.cond) / length(test.cond)
		
						ind.opt = which(cv.outRAW$rhos == cv.outRAW$bestrho)
						trErr.cv = cv.outRAW$errs[,ind.opt]
						accCV = 1 - sum(trErr.cv) / length(train.cond)
		
						cv.outRAW2 <- Classify.cv(trPLDA, train.cond, transform = TRUE, type = "deseq", folds = cv.outRAW$folds)
						outRAW2 <- Classify(trPLDA, train.cond, tsPLDA, rho=cv.outRAW2$bestrho, type = "deseq")
						ts.accPDA2 <- sum(outRAW2$ytehat == test.cond) / length(test.cond)
		
						ind.opt2 = which(cv.outRAW2$rhos == cv.outRAW2$bestrho)
						trErr2.cv = cv.outRAW2$errs[,ind.opt2]
						accCV2 = 1 - sum(trErr2.cv) / length(train.cond)
						
						##### NBLDA
						trNB = trPLDA
						y.trNB = train.cond
						
						tt = getT(t(trNB), sizeFactors = rep(1, nrow(trNB)))$target
						rM = rowMeans(t(trNB))
						rV = rowVars(t(trNB))
						
						disp = (rV - rM)/rM^2
						disp0 = numeric()
						
						for(i in 1:length(disp)){
							a1 = 0; a2 = disp[i]
							disp0[i] = max(a1,a2)
						}
						
						disperhat = getAdjustDisp(disp0, shrinkTarget = tt)$adj
						out = classnb(trNB, y.trNB, trNB, type = "deseq")
						out.ts = classnb(trNB, y.trNB, tsPLDA, type = "deseq")
						trAccNB.train = sum(out$ytehat == train.cond) / length(train.cond)
		
						tsAccNB.test = sum(out.ts$ytehat == test.cond) / length(test.cond)
					})
			 
					######################
					
					idxToWrite = which(testACC[,"Repeat"] == R & testACC[,"Transform"] == idxTransform)
				
					testACC[idxToWrite, "SVM"] = ts.accSVM
					testACC[idxToWrite, "RF"] = ts.accRF
					testACC[idxToWrite, "CART"] = ts.accCART
					testACC[idxToWrite, "PLDA"] = ts.accPDA
					testACC[idxToWrite, "PLDA2"] = ts.accPDA2
					testACC[idxToWrite, "bagSVM"] = svm.B$OveralAccuracy.Test
					testACC[idxToWrite, "NBLDA"] = tsAccNB.test
					
					if (class(modelSVM) == "train") trainACC_CV[idxToWrite, "SVM"] = max(modelSVM$results$Accuracy) else trainACC_CV[R, "SVM"] = NA
					if (class(modelRF) == "train") trainACC_CV[idxToWrite, "RF"] = max(modelRF$results$Accuracy) else trainACC_CV[R, "RF"] = NA
					if (class(modelCART) == "train") trainACC_CV[idxToWrite, "CART"] = max(modelCART$results$Accuracy) else trainACC_CV[R, "CART"] = NA
					trainACC_CV[idxToWrite, "PLDA"] = accCV
					trainACC_CV[idxToWrite, "PLDA2"] = accCV2
					trainACC_CV[idxToWrite, "NBLDA"] = trAccNB.train
					trainACC_CV[idxToWrite, "bagSVM"] = svm.B$OveralAccuracy.Train
					
					if (class(modelSVM) == "train") svmTune = modelSVM$bestTune else svmTune = NA
					if (class(modelCART) == "train") cartTune = modelCART$bestTune else cartTune = NA
					if (class(modelRF) == "train") rfTune = modelRF$bestTune else rfTune = NA
					
					tunes.rpt = cbind(idxTransform, R, svmTune, cartTune, rfTune)
					tunes = rbind(tunes, tunes.rpt)
					tunes.rpt = NULL
            }
        }
        
        trainACC_CV = try({cbind(combs.sub[m,], p[p.ind], trainACC_CV)}, silent = TRUE)
        testACC = try({cbind(combs.sub[m,], p[p.ind], testACC)}, silent = TRUE)
        tunes = try({cbind(combs.sub[m,], p[p.ind], tunes)}, silent = TRUE)
        
        tunes_m[[p.ind]] = tunes
        testACC_m[[p.ind]] = testACC  
        trainACC_CV_m[[p.ind]] = trainACC_CV 
        names(testACC_m)[p.ind] = names(trainACC_CV_m)[p.ind] = names(tunes_m)[p.ind] = paste("p", p[p.ind], sep="")  
    }
    
    ResultTest.rpt[[m]] = testACC_m
    ResultTrainCV.rpt[[m]] = trainACC_CV_m
    TuneParameters.rpt[[m]] = tunes_m
    names(TuneParameters.rpt)[m] = names(ResultTest.rpt)[m] = names(ResultTrainCV.rpt)[m] = names(TuneParameters.rpt)[m] = paste(c(names(combs.sub)[-1]), c(combs.sub[m,-1]), sep="", collapse = "")
    
    if (save){
        #if (transform == "rlog") {
        #    folders = dir()
        #    if (!any(folders == "RLOG")) dir.create("RLOG", recursive = TRUE)
        #    setwd("RLOG")
        #}
        
        #if (transform == "vst") {
        #    folders = dir()
        #    if (!any(folders == "VST")) dir.create("VST", recursive = TRUE)
        #    setwd("VST")
        #}
        
        folders = dir()
        if (!any(folders == "VST_RLOG")) dir.create("VST_RLOG", recursive = TRUE)
        setwd("VST_RLOG")
        
        save(ResultTest.rpt, file = "TestResults.rda")
        save(ResultTrainCV.rpt, file = "CVResults.rda")
        save(TuneParameters.rpt, file = "TuneParameters.rda")
        
        halt = paste("Algorithm stopped at: ", m, sep = "")
        write.table(halt, file = "stopped.txt", quote = FALSE, row.names = FALSE, col.names = FALSE)
    }
    setwd(wd)
}

end.time = Sys.time()
elapsed =  end.time - start.time
elapsed   ## elapsed time for simulation.


